Unit OExoTab;
{ gestion des fichiers d'exercices de ExoCalc }
{ K.B. novembre 1998 }

INTERFACE
Uses
 Classes, SysUtils, Winprocs, Utils;

Const
 {codes d'erreurs}
 erInitFile = 234;     {erreur  l'ouverture }
 erReadFile = 235;     {erreur de lecture }
 erFormatFile = 236;   {erreur de format }
 erNombre = 237;       {nombre mal crit }

 MaxExos = 20;         {nombre d'exercices  ne pas dpasser }

Type
 TStr80 = String[80];

 { enregistrement reprsentant un exercice }
 TExoRec = class
  Enonce  : TStringList;  { nonc de l'exercice }
  Reponse : Real;         { rponse attendue }
  Figure  : TStringList;  { figure Imageo }
  Image   : String;       { image BMP }
  Titre   : String;       { titre de l'exercice }
  Aide    : TStringList;  { aide contextuelle }
  Constructor Create;
  Destructor Destroy; override;
  End;

 { tableau contenant les exercices }
 TExoTab = class(TList)
  Cours : String;      { fichier Cours associ }
  Melange : Boolean;   { mlanger les exercices ? }
  AFaire : Integer;    { nombre d'exercices  faire }
  ErrorFlag : Integer; { indicateur d'erreur }
  ExoPath : String;    { rpertoire de l'exercice }
  { constructeur }
  Constructor Create;
  { destructeur }
  Destructor Destroy; override;
  Procedure LireFichier(NomFichier:String);
  { messages d'erreurs }
  Function GetErrorMsg:String;
  Function GetExorec(num:Integer):TExoRec;
  End;

IMPLEMENTATION

var
 {tableau utilis pour les mlanges;
  on suppose qu'il n'y aura pas plus de 21 exercices  mlanger}
 MelTab:array[0..MaxExos] of Integer;

Procedure Decompose(S:String; var S1,S2:TStr80; sep:Char);
{ coupe S en deux parties S1 et S2 autour de sep; supprime les espaces }
Var i:Integer;
Begin
 i:=pos(sep,S);
 if i=0 then
  begin
   S1:=''; S2:=''; exit;
  end;
 S1:=copy(S,1,i-1);
 S2:=copy(S,i+1,length(S)-i);
 repeat
  i:=pos(' ',S1);
  if i<>0 then delete(S1,i,1);
 until i=0;
 repeat
  i:=pos(' ',S2);
  if i<>0 then delete(S2,i,1);
 until i=0;
End;

procedure InitMelTab(var total:Integer; melange:Boolean);
{mlange des exercices si ncessaire }
var i,r,tmp :Integer;
begin
 if Total>Maxexos+1 then Total:=MaxExos+1;
 {on initialise les numros dans l'ordre }
 for i:=0 to total-1 do MelTab[i]:=i;
 if melange then {il faut mlanger }
  begin
   for i:=0 to total-1 do
    begin
     r:=random(total);
     tmp:=MelTab[i];
     MelTab[i]:=MelTab[r];
     MelTab[r]:=tmp;
    end;
  end;
end;

{ objet TExoRec }

constructor TExoRec.Create;
{initialisation d'un exercice : on donne des valeurs par dfaut
  tous les champs }
begin
 inherited Create;
 Reponse:=0; Image:=''; Titre:='';
 Enonce:=TStringList.Create;
 Figure:=TStringList.Create;
 Aide:=TStringList.Create;
end;

destructor TExoRec.Destroy;
{ destructionn d'un exercice : on libre la mmoire }
begin
 if Figure<>nil then Figure.Free;
 if Enonce<>nil then Enonce.Free;
 if Aide<>nil then Aide.Free;
 inherited Destroy;
end;

{ objet TExoTab : tableau contenant les exercices }

constructor TExoTab.Create;
{initialisation de la liste des exercices }
begin
 inherited Create;
 Cours:='';
 Melange:=true;
 AFaire:=0;
 ErrorFlag:=0;
 ExoPath:='';
end;

destructor TExoTab.Destroy;
{ destruction de la liste des exercices : on dtruit d'abord
  chacun des exercices }
var i:Integer;
    exorec:TExorec;
begin
 for i:=0 to Count-1 do
  begin
   exorec:=Items[i];
   if exorec<>nil then exorec.destroy;
  end;
 inherited Destroy;
end;

Function TExoTab.GetErrorMsg:String;
{ messages d'erreurs }
Begin
 case ErrorFlag of
  erInitFile  : GetErrorMsg:='Ouverture du fichier impossible.';
  erReadFile  : GetErrorMsg:='Erreur en lecture du fichier.';
  erFormatFile : GetErrorMsg:='Format de fichier non valide.';
  erNombre : GetErrorMsg:='Nombre mal crit.';
  else GetErrorMsg:='';
  end;
End;

function TExoTab.GetExorec(num:Integer):TExoRec;
{ donne l'exercice de numro num en tenant compte d'un ventuel mlange }
begin
 Result:=TExoRec(Items[Meltab[num]]);
end;

procedure TExoTab.LireFichier(NomFichier:String);
{ lecture du fichier d'exercices }
Var
 f:Text;         { le fichier texte }
 S:String;       { la ligne courante }
 ok:Boolean;     { indique si tout va bien }
 erec:TExoRec;   { un exercice }

 Procedure LigneSuivante;
 { lecture de la ligne suivante : les lignes vides et les lignes
   commenant par ; sont passes
   le rsultat se trouve dans S
    la fin du fichier S='' autrement S<>'' }
 Begin
  repeat
   if eof(f) then S:=''
   else
    begin
     {$I-} readln(f,S); {$I+}
     if IOResult<>0 then
      begin
       ok:=false;
       ErrorFlag:=erReadFile;
       S:='';
      end
     else
      begin
       { remplacer les commentaires par des lignes vides }
       if (S<>'') and (S[1]=';') then S:=''
       else
        { supprimer les espaces de fin de ligne }
        while (S<>'') and (S[length(S)]=' ') do dec(S[0]);
      end;
    end;
  until (S<>'') or eof(f);
  Asci2Ansi(S);
 End;

 Procedure LireEntete;
 { lecture de l'entte du fichier }
 Var
  S1,S2 : TStr80;
  n,err : Integer;
 Begin
  LigneSuivante;
  { la premire ligne indique l'entte }
  if S<>'[Fichier EXO]' then
   begin
    ok:=false;
    exit;
   end;
  { renseignements contenus dans l'entte }
  repeat
   LigneSuivante;
   Decompose(S,S1,S2,'=');
   if S1='Nombre' then
    begin
     val(S2,n,err);
     if err=0 then AFaire:=n;
    end
   else if S1='Mlange' then
    begin
     if S2='oui' then Melange:=true
     else Melange:=false;
    end
   else if S1='Cours' then
    begin
     if S2<>'' then Cours:=S2;
    end;
  until (S='[EXERCICE]') or (S='');
  { aprs l'entte on arrive au premier exercice }
  if S='' then ok:=false;
 End;

 Procedure LireReponse(var r:Real);
 { lecture du nombre donnant la rponse }
 Var err:Integer;
 Begin
  LigneSuivante;
  val(S,r,err);
  if err<>0 then { nombre mal crit }
   begin
    ok:=false;
    ErrorFlag:=erNombre;
   end
  else LigneSuivante;
 End;

 Function LireChaine:String;
 { lecture d'une chaine de caractres simple pour
   le titre et le nom de fichier PCX }
 Begin
  LigneSuivante;
  Result:=S;
  LigneSuivante;
  if S='' then ok:=false;
 End;

 Procedure LirePara(ps:TStringList);
 { lecture d'un paragraphe pour l'nonc, la figure et l'aide }
 Begin
  LigneSuivante;
  { le caractre & indique le dbut d'une nouvelle section }
  while ok and (S<>'') and (S[1]<>'&') and (S<>'[EXERCICE]') do
   begin
    ps.Add(S);
    LigneSuivante;
   end;
 End;

 Function LireExercice:TExoRec;
 { lecture des donnes relatives  un exercice }
 Var exorec:TExorec;
 Begin
  { la ligne [EXERCICE] vient d'tre lue }
  { cration d'un enregistrement vide et initialisation }
  exorec:=TExoRec.Create;
  LigneSuivante;
  { tant que tout va bien, que le fichier n'est pas fini et qu'un
    nouvel exercice ne commence pas on lit les sections d'un
    exercice }
  while ok and (S<>'') and (S<>'[EXERCICE]') do
   begin
    if S='&ENONCE' then
     LirePara(ExoRec.Enonce)
    else if S='&REPONSE' then
     LireReponse(Exorec.Reponse)
    else if S='&TITRE' then
     Exorec.Titre:=LireChaine
    else if S='&IMAGE' then
     Exorec.Image:=LireChaine
    else if S='&FIGURE' then
     LirePara(Exorec.Figure)
    else if S='&AIDE' then
     LirePara(Exorec.Aide)
    else ok:=false;
   end;
  LireExercice:=exorec;
 End;

Begin
 { ouverture du fichier de donnes }
 assign(f,NomFichier);
 {$I-} reset(f); {$I+}
 if IOResult<>0 then
  begin
   ErrorFlag:=erInitFile; exit;
  end;
 { ok indique que tout va bien }
 ok:=true;
 ExoPath:=ExtractFilePath(ExpandFileName(NomFichier));
 { lecture de l'entte }
 LireEntete;
 { tant qu'on trouve un exercice on le lit }
 while ok and (S='[EXERCICE]') do
  begin
   erec:=LireExercice;
   Add(erec);
  end;
 { y a-t-il eu une erreur dans le fichier }
 if not ok then
  if ErrorFlag=0 then ErrorFlag:=erFormatFile;
 { fermeture du fichier }
 {$I-} close(f); {$I+}
 if (Afaire=0) or (Afaire>Count) then Afaire:=Count;
 InitMelTab(Afaire,Melange);
End;

END.

{                       Fin du fichier OExoTab.Pas                          }

